@@ -1897,11 +1897,8 @@ InitFonts (void)
     } else 
 #endif
     {
-#ifdef BUILTIN_FONTS
         BuiltinRegisterFpeFunctions();
-#else
 	FontFileRegisterFpeFunctions();
-#endif
 #ifndef NOFONTSERVERACCESS
 	fs_register_fpe_functions();
 #endif
