@@ -102,6 +102,12 @@
 #define	PRIM_OPS_NO_REDEFINE_ASM
 #include "x86emu/x86emui.h"
 
+#if defined(__GNUC__)
+# if defined (__i386__) || defined(__i386) || defined(__AMD64__) || defined(__x86_64__) || defined(__amd64__)
+#  include "x86emu/prim_x86_gcc.h"
+# endif
+#endif
+
 /*------------------------- Global Variables ------------------------------*/
 
 static u32 x86emu_parity_tab[8] =
@@ -2654,3 +2660,41 @@ DB(	if (CHECK_SP_ACCESS())
     return res;
 }
 
+/****************************************************************************
+REMARKS:
+CPUID takes EAX/ECX as inputs, writes EAX/EBX/ECX/EDX as output
+****************************************************************************/
+void cpuid (void)
+{
+    u32 feature = M.x86.R_EAX;
+#ifdef X86EMU_HAS_HW_CPUID
+        hw_cpuid(&M.x86.R_EAX, &M.x86.R_EBX, &M.x86.R_ECX, &M.x86.R_EDX);
+#endif
+    switch (feature) {
+    case 0:
+        M.x86.R_EAX = 1;		// maximum function number we support
+#ifndef X86EMU_HAS_PRIM_CPUID
+        M.x86.R_EBX = 0x756e6547;
+        M.x86.R_ECX = 0x6c65746e;
+        M.x86.R_EDX = 0x49656e69;
+#endif
+        break;
+    case 1:
+#ifndef X86EMU_HAS_PRIM_CPUID
+        M.x86.R_EAX = 0x00000480;	// 486dx4
+        M.x86.R_EBX = 0x00000000;
+        M.x86.R_ECX = 0x00000000;
+        M.x86.R_EDX = 0x00000002;	// VME
+#else
+        M.x86.R_EDX &= 0x00000012;	// TSC and VME
+#endif
+        break;
+    default:
+        M.x86.R_EAX = 0;		// don't support extended features
+        M.x86.R_EBX = 0;
+        M.x86.R_ECX = 0;
+        M.x86.R_EDX = 0;
+        break;
+    }
+}
+
