@@ -1,4 +1,4 @@
 #define MFBPOLYGLYPHBLT xf1bppPolyGlyphBltInvert
 #define OPEQ ^=
 #include "mfbmap.h"
-#include "../../../../mfb/mfbplygblt.c"
+#include "../../../mfb/mfbplygblt.c"
